clear
close all

syms u v

% inpute surface is given as a height field
% we use a symbolic expression to get exact derivatives from Matlab
h = 0.5*(0.25*u^2 - 0.4*v^2 + 0.4*u*v + cos(u) - 2*sin(u*v));

figure
fsurf(u, v, h, [-2 2 -2 2]);

hold on
axis equal
axis vis3d
rotate3d

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% There are easy formulas for height field surface to compute
% K and H, see e.g. do Carmo
hx = diff(h, u);
hy = diff(h, v);

hxx = diff(h, u, 2);
hxy = diff(diff(h, u), v);
hyy = diff(h, v, 2);

K = (hxx*hyy-hxy^2)/((1+hx^2+hy^2)^2);

% Gaussian curvature plot (as height field)
figure
fsurf(u, v, K, [-2 2 -2 2]);

axis equal
axis vis3d
rotate3d

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% to display color we convert from symbolic to numeric 
% representation
x = linspace(-2, 2, 30);
y = linspace(-2, 2, 30);

[X, Y] = meshgrid(x,y);
 
Z = double(subs(h, {u, v}, {X, Y}));
G = double(subs(K, {u, v}, {X, Y}));
C = zeros(size(Z,1), size(Z,2), 3);

Gmax = max(max(G));
Gmin = min(min(G));

c0 = [0, 0, 1]; % blue  - negative curvature
c1 = [0, 1, 0]; % green - zero
c2 = [1, 0, 0]; % red   - positive curvature

[n, m] = size(G);

for i = 1:n
    for j = 1:m
        if G(i,j) < 0
            %if G(i,j) < Gmin, G(i,j) = Gmin; end
            C(i,j,:) = ((Gmin-G(i,j))/Gmin) * c1 + (G(i,j)/Gmin) * c0;
        else
            %if G(i,j) > Gmax, G(i,j) = Gmax; end
            C(i,j,:) = ((Gmax-G(i,j))/Gmax) * c1 + (G(i,j)/Gmax) * c2;
        end
    end
end

figure

% plot surface, color according to Gaussian curvature
surf(X, Y, Z, C, 'FaceColor', 'interp');

axis equal
axis vis3d
rotate3d


