clear
close all

% load a point set
P = load('P.txt', '-ascii');
n = size(P, 1);

% space for color values
C = zeros(size(P));

K = zeros(n, 1);
H = zeros(n, 1);

E = zeros(4*n, 6);
E(:, 1:3) = repelem(P, 4, 1);

nn = 6;

for i = 1:n
    % we need at least 6 neighbors for the fit
    r = 0.5;
    m = 0;
    while m < nn
        I = rangesearch(P, P(i, :), r);
        m = length(I{1});
        r = 1.5*r;
    end
    Q = P(I{1}, :);

    % perform PCA
    [~, V] = pca(Q);

    % now fit with a paraboloid
    V = V(:, [3 2 1]);
    B = P(i, :)';
    c = jetfit(Q, B, V(:,1), V(:,2), V(:,3));

    % estimate curvature
    [D, k] = curvest(c, 0, 0);
    E(4*i-3:4*i-2, 4:end) = (V*D)';
    E(4*i-1:4*i, 4:end) = -(V*D)';
    
    % compute and store Gaussian/mean curvature
    K(i) = k(1)*k(2);
    H(i) = 0.5*(k(1)+k(2));
end

% determine min/max of curvature range
f = 0.98;
Kmax = f*max(K);
Kmin = f*min(K);

c0 = [0, 0, 1];
c1 = [0, 1, 0];
c2 = [1, 0, 0];

for i = 1:n
    if K(i) < 0
        if K(i) < Kmin, K(i) = Kmin; end
        C(i, :) = ((Kmin-K(i))/Kmin) * c1 + (K(i)/Kmin) * c0;
    else
        if K(i) > Kmax, K(i) = Kmax; end
        C(i, :) = ((Kmax-K(i))/Kmax) * c1 + (K(i)/Kmax) * c2;
    end
end

% display point set
scatter3(P(:,1), P(:,2), P(:,3), 35, C, 'filled');
hold on
quiver3(E(:,1), E(:,2), E(:,3), E(:,4), E(:,5), E(:,6), 'k');

hold on
axis equal
axis vis3d

xlabel('X');
ylabel('Y');

% switch to rotate mode
rotate3d