% press ctrl-c in the console window to stop the random selection
% of points

clear
close all

% load a point set
P = load('P.txt', '-ascii');

% display point set
scatter3(P(:,1), P(:,2), P(:,3), 'MarkerEdgeColor', 'k', ...
                                 'MarkerFaceColor', [0 .75 .75]);

hold on
axis equal
axis vis3d

xlabel('X');
ylabel('Y');

% switch to rotate mode
rotate3d

while 1
    % pick a random point from P
    n = size(P, 1);
    k = randi(n, 1);

    % we need at least 6 neighbors for the fit
    r = 0.5;
    I = rangesearch(P, P(k, :), r);
    Q = P(I{1}, :);

    % display neighborhood point set
    scatter3(Q(:,1), Q(:,2), Q(:,3), 'MarkerEdgeColor', 'k', ...
                                     'MarkerFaceColor', 'y');
    % perform PCA
    [~, V] = pca(Q);
    B = P(k, :)';
    
    % show result
    scatter3(B(1), B(2), B(3), 'MarkerEdgeColor', 'k', ...
                               'MarkerFaceColor', 'r');
                       
    X = repmat(B', 3, 1);
    quiver3(X(:,1), X(:,2), X(:,3), V(1,:)', V(2,:)', V(3,:)', 'b', 'LineWidth', 2);

    % now fit with a paraboloid
    V = V(:, [3 2 1]);
    c = jetfit(Q, B, V(:,1), V(:,2), V(:,3));

    % plot the resulting paraboloid
    syms u v

    X = B + u*V(:,1) + v*V(:,2) + (c(1)*u*u + c(2)*u*v + c(3)*v*v + ...
                                   c(4)*u   + c(5)*v   + c(6)    )*V(:,3);
                           
    fsurf(X(1), X(2), X(3), [-r r -r r], 'FaceColor', [0.5, 0.5, 0.5])

    id = waitforbuttonpress;
end
